import atexit
import datetime as dt
import json
import logging
import logging.config
import os
from pathlib import Path
from typing import override

PROJECT_ROOT = Path(__file__).parent.parent
if LOG_DIR := os.getenv("LOG_DIR"):
    LOG_DIR = Path(LOG_DIR)
    LOG_DIR.mkdir(parents=True, exist_ok=True)

LOG_RECORD_BUILTIN_ATTRS = {
    "args",
    "asctime",
    "created",
    "exc_info",
    "exc_text",
    "filename",
    "funcName",
    "levelname",
    "levelno",
    "lineno",
    "module",
    "msecs",
    "message",
    "msg",
    "name",
    "pathname",
    "process",
    "processName",
    "relativeCreated",
    "stack_info",
    "thread",
    "threadName",
    "taskName",
}


def setup_logging(loglevel: int | str | None = None):
    """
    Setup logging configuration
    :return:
    """
    config_file = PROJECT_ROOT / "conf" / "logging_config.json"
    with open(config_file, "r") as file:
        config = json.load(file)

    # Override log level if provided
    if loglevel:
        level_str = logging.getLevelName(loglevel)
        if not isinstance(level_str, str):
            level_str = logging.getLevelName(level_str)
        for handler in config["handlers"].values():
            handler["level"] = level_str

    # Set log file path to user log directory
    if LOG_DIR:
        config["handlers"]["file"]["filename"] = LOG_DIR / "adGroupSync.log.jsonl"

    # Create path and file if it does not exist
    Path(config["handlers"]["file"]["filename"]).parent.mkdir(
        parents=True, exist_ok=True)
    Path(config["handlers"]["file"]["filename"]).touch()

    logging.config.dictConfig(config)
    queue_handler = logging.getHandlerByName("queue_handler")
    if queue_handler is not None:
        queue_handler.listener.start()
        atexit.register(queue_handler.listener.stop)


class JSONFormatter(logging.Formatter):
    """
    A custom JSON formatter for logging
    """
    def __init__(
            self,
            *,
            fmt_keys: dict[str, str] | None = None,
    ):
        super().__init__()
        self.fmt_keys = fmt_keys if fmt_keys is not None else {}

    @override
    def format(self, record: logging.LogRecord) -> str:
        message = self._prepare_log_dict(record)
        return json.dumps(message, default=str)

    def _prepare_log_dict(self, record: logging.LogRecord) -> dict:
        always_fields = {
            "message": record.getMessage(),
            "timestamp": dt.datetime.fromtimestamp(
                record.created, tz=dt.timezone.utc
            ).isoformat()
        }
        if record.exc_info is not None:
            always_fields["exc_info"] = self.formatException(record.exc_info)

        if record.stack_info is not None:
            always_fields["stack_info"] = self.formatStack(record.stack_info)

        message = {
            key: msg_val
            if (msg_val := always_fields.pop(val, None)) is not None
            else getattr(record, val)
            for key, val in self.fmt_keys.items()
        }
        message.update(always_fields)

        # Include all other attributes
        for key, val, in record.__dict__.items():
            if key not in LOG_RECORD_BUILTIN_ATTRS:
                message[key] = val

        return message
