import logging
import os
from datetime import datetime
from pathlib import Path

import pytz
from ldap3 import SIMPLE
from ms_active_directory import ADDomain, ADGroup, ADUser, ADObject

from .models import RecentRun

DOMAIN = os.getenv('AD_DOMAIN')
USER_NAME = os.getenv('AD_USER')
PASSWORD = os.getenv('AD_PASSWORD')
LDAP_SERVER = [s.strip() for s in os.getenv('AD_LDAP_SERVER').split(',')]
TIMEZONE = pytz.timezone(os.getenv('TIMEZONE', 'UTC'))
MAILINGLISTS_PARENT_GROUP = os.getenv('AD_MAILINGLISTS_PARENT_GROUP')

# Setup logging
logger = logging.getLogger('adGroupSync')
logger.setLevel(os.getenv('LOG_LEVEL', logging.INFO))


def has_changed(object_: ADObject, recent_run):
    """
    Check if the object has changed since the last run
    :param object_: Object to check
    :param recent_run: RecentRun object
    :return: True if the object has changed, False otherwise
    """
    modify_timestamp = object_.get('modifyTimestamp')
    if modify_timestamp is None:
        message = (f"Object '{object_.get('sn')}' of type '{type(object_)}' "
                   "does not have 'modifyTimestamp' attribute")
        logger.error(message)
        raise ValueError(message)
    return modify_timestamp > recent_run


def main():

    logger.info("Running group sync")

    # Get the last run timestamp
    recent_run = RecentRun(Path().home() / '.recent_run')
    logger.info(f"Last run: {recent_run.datetime.strftime('%Y-%m-%d %H:%M:%S %Z')}")

    # Setup session
    domain = ADDomain(DOMAIN, ldap_servers_or_uris=LDAP_SERVER)
    logger.debug(f"Domain: {domain}", extra={'domain': domain})
    try:
        session = domain.create_session_as_user(
            USER_NAME, PASSWORD, authentication_mechanism=SIMPLE)
    except Exception as e:
        logger.error(f"Error creating session: {e}")
        exit(1)
    logger.debug(f"Session: {session}")
    logger.info(f"Session opened: {session.is_open()}")

    # Get parent group
    parent_group = session.find_group_by_name(
        MAILINGLISTS_PARENT_GROUP, ['modifyTimestamp'])
    parent_group_has_changed = has_changed(parent_group, recent_run)
    logger.debug(f"Parent group: {parent_group}", extra={
        'group': parent_group,
        'modifyTimestamp': parent_group.get('modifyTimestamp'),
        'has_changed': parent_group_has_changed})

    # Get child groups
    group_attrs = [
        'modifyTimestamp',
        'objectSid',
        'mail',
        'sAMAccountName',
    ]
    group_members = session.find_members_of_group_recursive(
        parent_group, group_attrs)[1]
    groups = (m for m in group_members.values() if isinstance(m, ADGroup))
    logger.debug(f"Found {len(groups)} child groups",
                 extra={'child_group_count': len(groups),
                        'child_groups': groups})
    users = (m for m in groups if not isinstance(m, ADUser))

    # If the parent group has changed, we need to update the child groups
    # in CiviCRM
    if parent_group_has_changed:
        civicrm.update_mailing_lists(groups)

    # Check if the mailing list groups have changed and update them in CiviCRM
    for ml_group in groups:
        if has_changed(ml_group, recent_run):
            civicrm.update_mailing_list_members(ml_group)

    # Check if the users have changed and update them in CiviCRM
    for user in users:
        if has_changed(user, recent_run):
            civicrm.update_user(user)

    # Set the last run timestamp
    recent_run.datetime = datetime.now(tz=TIMEZONE)

if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        logger.error(f"Error: {e}")
        exit(1)


