from datetime import datetime, tzinfo
from pathlib import Path
import pytz


class RecentRun:
    """
    Class to manage the last run of the script
    """

    def __init__(self, file_path: Path, timezone: tzinfo = pytz.utc):
        """
        Initialize the class
        :param file_path: File path to store the last run timestamp
        :param timezone: Timezone to use for the timestamp
        """
        self._datetime = None
        self._timezone = timezone
        self._file_path = file_path

        if not self._file_path.exists():
            raise FileNotFoundError(f"File {self._file_path} does not exist")

        self._read_recent_run_from_file()

    def _write_recent_run_to_file(self):
        """
        Write the recent run timestamp to the file
        :return:
        """
        with open(self._file_path, 'w') as f:
            f.write(str(self.timestamp))

    def _read_recent_run_from_file(self):
        """
        Read the recent run timestamp from the file
        :return:
        """
        with open(self._file_path, 'r') as f:
            line = f.readline()
            if not line or line.strip() == '':
                return None
            try:
                self._datetime = datetime.fromtimestamp(float(line), tz=self._timezone)
            except ValueError:
                raise ValueError(f"Invalid timestamp '{line}' in {self._file_path}")

    @property
    def datetime(self):
        return self._datetime

    @property
    def timestamp(self):
        return self._datetime.timestamp()

    @datetime.setter
    def datetime(self, value: datetime):
        if value.tzinfo is None:
            value = value.astimezone(self._timezone)
        self._datetime = value
        self._write_recent_run_to_file()

    @staticmethod
    def _to_datetime(value: datetime|str|float):
        """
        Convert the value to a datetime object
        :param value:
        :return:
        """
        try:
            if isinstance(value, str):
                value = float(value)
            if isinstance(value, float):
                value = datetime.fromtimestamp(value)
        except ValueError:
            raise ValueError(f"Invalid timestamp '{value}'")
        return value

    def __gt__(self, other: datetime|str|float):
        return self.datetime > self._to_datetime(other)

    def __lt__(self, other: datetime|str|float):
        return self.datetime < self._to_datetime(other)

    def __eq__(self, other: datetime|str|float):
        return self.datetime == self._to_datetime(other)

    def __ge__(self, other: datetime|str|float):
        return self.datetime >= self._to_datetime(other)

    def __le__(self, other: datetime|str|float):
        return self.datetime <= self._to_datetime(other)
